/* Copyright (C) 2013 by Bundesamt fuer Strahlenschutz
 * Software engineering by Intevation GmbH
 *
 * This file is Free Software under the GNU GPL (v>=3) 
 * and comes with ABSOLUTELY NO WARRANTY! Check out 
 * the documentation coming with IMIS-Labordaten-Application for details. 
 */
package de.intevation.lada;

import java.util.ArrayList;

import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import org.apache.log4j.Logger;
import org.jboss.arquillian.junit.Arquillian;
import org.jboss.arquillian.persistence.Cleanup;
import org.jboss.arquillian.persistence.CleanupStrategy;
import org.jboss.arquillian.persistence.DataSource;
import org.jboss.arquillian.persistence.TestExecutionPhase;
import org.jboss.arquillian.persistence.UsingDataSet;
import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;

import de.intevation.lada.importer.Identified;
import de.intevation.lada.importer.Identifier;
import de.intevation.lada.importer.IdentifierConfig;
import de.intevation.lada.model.land.Messung;
import de.intevation.lada.model.land.Probe;

/**
 * Class to test the Lada-Importer
 *
 * @author <a href="mailto:rrenkert@intevation.de">Raimund Renkert</a>
 */
@RunWith(Arquillian.class)
public class ImporterTest extends BaseTest{

    @Inject
    Logger internalLogger;

    @PersistenceContext(unitName="land")
    EntityManager em;

    @Inject
    @IdentifierConfig(type="Probe")
    Identifier probeIdentifier;

    @Inject
    @IdentifierConfig(type="Messung")
    Identifier messungIdentifier;

    public ImporterTest() {
        testProtocol = new ArrayList<Protocol>();
    }

    /**
     * PROBE IDENTIFIER
     */
    @Test
    @UsingDataSet("datasets/dbUnit_probe_import.json")
    @DataSource("java:jboss/lada-land")
    @Cleanup(phase=TestExecutionPhase.AFTER,
        strategy=CleanupStrategy.USED_TABLES_ONLY)
    public final void identifyProbeByHPNrMST() throws Exception {
        Protocol protocol = new Protocol();
        protocol.setName("import");
        protocol.setType("identify probe");
        protocol.addInfo("import", "Compare and find Probe by HP-Nr. and MST, Update");

        Probe probe = new Probe();
        probe.setHauptprobenNr("120510002");
        probe.setMstId("06010");

        Identified found = probeIdentifier.find(probe);
        Assert.assertEquals(Identified.UPDATE, found);
        protocol.setPassed(true);
        testProtocol.add(protocol);
    }

    @Test
    @UsingDataSet("datasets/dbUnit_probe_import.json")
    @DataSource("java:jboss/lada-land")
    @Cleanup(phase=TestExecutionPhase.AFTER,
        strategy=CleanupStrategy.USED_TABLES_ONLY)
    public final void identifyProbeByHPNrMSTNew() throws Exception {
        Protocol protocol = new Protocol();
        protocol.setName("import");
        protocol.setType("identify probe");
        protocol.addInfo("import", "Compare and find Probe by HP-Nr. and MST, New");

        Probe probe = new Probe();
        probe.setHauptprobenNr("120510003");
        probe.setMstId("06010");

        Identified found = probeIdentifier.find(probe);
        Assert.assertEquals(Identified.NEW, found);
        protocol.setPassed(true);
        testProtocol.add(protocol);
    }

    @Test
    @UsingDataSet("datasets/dbUnit_probe_import.json")
    @DataSource("java:jboss/lada-land")
    @Cleanup(phase=TestExecutionPhase.AFTER,
        strategy=CleanupStrategy.USED_TABLES_ONLY)
    public final void identifyProbeByIdAlt() throws Exception {
        Protocol protocol = new Protocol();
        protocol.setName("import");
        protocol.setType("identify probe");
        protocol.addInfo("import", "Compare and find Probe by idAlt, Update");

        Probe probe = new Probe();
        probe.setIdAlt("T001");

        Identified found = probeIdentifier.find(probe);
        Assert.assertEquals(Identified.UPDATE, found);
        protocol.setPassed(true);
        testProtocol.add(protocol);
    }

    @Test
    @UsingDataSet("datasets/dbUnit_probe_import.json")
    @DataSource("java:jboss/lada-land")
    @Cleanup(phase=TestExecutionPhase.AFTER,
        strategy=CleanupStrategy.USED_TABLES_ONLY)
    public final void identifyProbeByIdAltNew() throws Exception {
        Protocol protocol = new Protocol();
        protocol.setName("import");
        protocol.setType("identify probe");
        protocol.addInfo("import", "Compare and find Probe by idAlt, New");

        Probe probe = new Probe();
        probe.setIdAlt("T002");

        Identified found = probeIdentifier.find(probe);
        Assert.assertEquals(Identified.NEW, found);
        protocol.setPassed(true);
        testProtocol.add(protocol);
    }

    @Test
    @UsingDataSet("datasets/dbUnit_probe_import.json")
    @DataSource("java:jboss/lada-land")
    @Cleanup(phase=TestExecutionPhase.AFTER,
        strategy=CleanupStrategy.USED_TABLES_ONLY)
    public final void identifyProbeByIdAltReject() throws Exception {
        Protocol protocol = new Protocol();
        protocol.setName("import");
        protocol.setType("identify probe");
        protocol.addInfo("import", "Compare and find Probe by idAlt, Reject");

        Probe probe = new Probe();
        probe.setIdAlt("T001");
        probe.setHauptprobenNr("120510003");
        probe.setMstId("06010");

        Identified found = probeIdentifier.find(probe);
        Assert.assertEquals(Identified.REJECT, found);
        protocol.setPassed(true);
        testProtocol.add(protocol);
    }

    @Test
    @UsingDataSet("datasets/dbUnit_probe_import.json")
    @DataSource("java:jboss/lada-land")
    @Cleanup(phase=TestExecutionPhase.AFTER,
        strategy=CleanupStrategy.USED_TABLES_ONLY)
    public final void identifyProbeByIdAltUpdate() throws Exception {
        Protocol protocol = new Protocol();
        protocol.setName("import");
        protocol.setType("identify probe");
        protocol.addInfo("import", "Compare and find Probe by idAlt, Update");

        Probe probe = new Probe();
        probe.setIdAlt("T001");
        probe.setHauptprobenNr("");
        probe.setMstId("06010");

        Identified found = probeIdentifier.find(probe);
        Assert.assertEquals(Identified.UPDATE, found);
        protocol.setPassed(true);
        testProtocol.add(protocol);
    }

    /**
     * MESSUNG IDENTIFIER
     */
    @Test
    @UsingDataSet("datasets/dbUnit_messung_import.json")
    @DataSource("java:jboss/lada-land")
    @Cleanup(phase=TestExecutionPhase.AFTER,
        strategy=CleanupStrategy.USED_TABLES_ONLY)
    public final void identifyMessungByNpNr() throws Exception {
        Protocol protocol = new Protocol();
        protocol.setName("import");
        protocol.setType("identify messung");
        protocol.addInfo("import", "Compare and find Messung by NP-Nr., Update");

        Messung messung = new Messung();
        messung.setProbeId(1000);
        messung.setNebenprobenNr("06A0");

        Identified found = messungIdentifier.find(messung);
        Assert.assertEquals(Identified.UPDATE, found);
        protocol.setPassed(true);
        testProtocol.add(protocol);
    }

    @Test
    @UsingDataSet("datasets/dbUnit_messung_import.json")
    @DataSource("java:jboss/lada-land")
    @Cleanup(phase=TestExecutionPhase.AFTER,
        strategy=CleanupStrategy.USED_TABLES_ONLY)
    public final void identifyMessungByNpNrNew() throws Exception {
        Protocol protocol = new Protocol();
        protocol.setName("import");
        protocol.setType("identify messung");
        protocol.addInfo("import", "Compare and find Messung by NP-Nr., New");

        Messung messung = new Messung();
        messung.setProbeId(1000);
        messung.setNebenprobenNr("06A1");

        Identified found = messungIdentifier.find(messung);
        Assert.assertEquals(Identified.NEW, found);
        protocol.setPassed(true);
        testProtocol.add(protocol);
    }

    @Test
    @UsingDataSet("datasets/dbUnit_messung_import.json")
    @DataSource("java:jboss/lada-land")
    @Cleanup(phase=TestExecutionPhase.AFTER,
        strategy=CleanupStrategy.USED_TABLES_ONLY)
    public final void identifyMessungByIdAlt() throws Exception {
        Protocol protocol = new Protocol();
        protocol.setName("import");
        protocol.setType("identify messung");
        protocol.addInfo("import", "Compare and find Messung by idAlt, Update");

        Messung messung = new Messung();
        messung.setProbeId(1000);
        messung.setIdAlt(1);

        Identified found = messungIdentifier.find(messung);
        Assert.assertEquals(Identified.UPDATE, found);
        protocol.setPassed(true);
        testProtocol.add(protocol);
    }

    @Test
    @UsingDataSet("datasets/dbUnit_messung_import.json")
    @DataSource("java:jboss/lada-land")
    @Cleanup(phase=TestExecutionPhase.AFTER,
        strategy=CleanupStrategy.USED_TABLES_ONLY)
    public final void identifyMessungByIdAltNew() throws Exception {
        Protocol protocol = new Protocol();
        protocol.setName("import");
        protocol.setType("identify messung");
        protocol.addInfo("import", "Compare and find Messung by idAlt, New");

        Messung messung = new Messung();
        messung.setProbeId(1000);
        messung.setIdAlt(2);

        Identified found = messungIdentifier.find(messung);
        Assert.assertEquals(Identified.NEW, found);
        protocol.setPassed(true);
        testProtocol.add(protocol);
    }

    @Test
    @UsingDataSet("datasets/dbUnit_messung_import.json")
    @DataSource("java:jboss/lada-land")
    @Cleanup(phase=TestExecutionPhase.AFTER,
        strategy=CleanupStrategy.USED_TABLES_ONLY)
    public final void identifyMessungByIdAltReject() throws Exception {
        Protocol protocol = new Protocol();
        protocol.setName("import");
        protocol.setType("identify messung");
        protocol.addInfo("import", "Compare and find Messung by idAlt, Reject");

        Messung messung = new Messung();
        messung.setProbeId(1000);
        messung.setIdAlt(1);
        messung.setNebenprobenNr("06A2");

        Identified found = messungIdentifier.find(messung);
        Assert.assertEquals(Identified.REJECT, found);
        protocol.setPassed(true);
        testProtocol.add(protocol);
    }

    @Test
    @UsingDataSet("datasets/dbUnit_messung_import.json")
    @DataSource("java:jboss/lada-land")
    @Cleanup(phase=TestExecutionPhase.AFTER,
        strategy=CleanupStrategy.USED_TABLES_ONLY)
    public final void identifyMessungByIdAltUpdate() throws Exception {
        Protocol protocol = new Protocol();
        protocol.setName("import");
        protocol.setType("identify messung");
        protocol.addInfo("import", "Compare and find Messung by idAlt, Update");

        Messung messung = new Messung();
        messung.setProbeId(1000);
        messung.setIdAlt(1);
        messung.setNebenprobenNr("");

        Identified found = messungIdentifier.find(messung);
        Assert.assertEquals(Identified.UPDATE, found);
        protocol.setPassed(true);
        testProtocol.add(protocol);
    }
}
